/*
 * $RCSfile: TIFFColorConverter.java,v $
 *
 * 
 * Copyright (c) 2005 Sun Microsystems, Inc. All  Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met: 
 * 
 * - Redistribution of source code must retain the above copyright 
 *   notice, this  list of conditions and the following disclaimer.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in 
 *   the documentation and/or other materials provided with the
 *   distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of 
 * contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any 
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND 
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL 
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR 
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
 * REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES. 
 * 
 * You acknowledge that this software is not designed or intended for 
 * use in the design, construction, operation or maintenance of any 
 * nuclear facility. 
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/28 01:01:59 $
 * $State: Exp $
 */
package com.sun.media.imageio.plugins.tiff;

/**
 * An abstract class that performs simple color conversion on 3-banded source
 * images, for use with the TIFF Image I/O plug-in.
 */
public abstract class TIFFColorConverter {

    /**
     * Constructs an instance of a <code>TIFFColorConverter</code>.
     */
    public TIFFColorConverter() {}

    /**
     * Converts an RGB triple into the native color space of this
     * TIFFColorConverter, and stores the result in the first three
     * entries of the <code>result</code> array.
     *
     * @param r the red value.
     * @param g the green value.
     * @param b the blue value.
     * @param result an array of <code>float</code>s containing three elements.
     * @throws NullPointerException if <code>result</code> is
     * <code>null</code>.
     * @throws ArrayIndexOutOfBoundsException if
     * <code>result.length&nbsp;<&nbsp;3</code>.
     */
    public abstract void fromRGB(float r, float g, float b, float[] result);

    /**
     * Converts  a   triple  in  the   native  color  space   of  this
     * TIFFColorConverter into an RGB triple, and stores the result in
     * the first three entries of the <code>rgb</code> array.
     *
     * @param x0 the value of channel 0.
     * @param x1 the value of channel 1.
     * @param x2 the value of channel 2.
     * @param rgb an array of <code>float</code>s containing three elements.
     * @throws NullPointerException if <code>rgb</code> is
     * <code>null</code>.
     * @throws ArrayIndexOutOfBoundsException if
     * <code>rgb.length&nbsp;<&nbsp;3</code>.
     */
    public abstract void toRGB(float x0, float x1, float x2, float[] rgb);
}
